@@ -1,85 +1,93 @@
 # This file is part of ranger, the console file manager.
 # License: GNU GPL version 3, see the file "AUTHORS" for details.
 
-CONTAINER_EXTENSIONS = ('7z', 'ace', 'ar', 'arc', 'bz', 'bz2', 'cab', 'cpio',
-    'cpt', 'deb', 'dgc', 'dmg', 'gz', 'iso', 'jar', 'msi', 'pkg', 'rar',
-    'shar', 'tar', 'tbz', 'tgz', 'xar', 'xpi', 'xz', 'zip')
-DOCUMENT_EXTENSIONS = ('cfg', 'css', 'cvs', 'djvu', 'doc', 'docx', 'gnm',
-    'gnumeric', 'htm', 'html', 'md', 'odf', 'odg', 'odp', 'ods', 'odt', 'pdf',
-    'pod', 'ps', 'rtf', 'sxc', 'txt', 'xls', 'xlw', 'xml', 'xslx')
-DOCUMENT_BASENAMES = ('bugs', 'bugs', 'changelog', 'copying', 'credits',
-    'hacking', 'help', 'install', 'license', 'readme', 'todo')
-
-BAD_INFO = '?'
+from __future__ import (absolute_import, division, print_function)
 
 import re
 from grp import getgrgid
-from os import lstat, stat, getcwd
-from os.path import abspath, basename, dirname, realpath, splitext, extsep, relpath
+from os import lstat, stat
+from os.path import abspath, basename, dirname, realpath, relpath
 from pwd import getpwuid
-from ranger.core.linemode import *
+from time import time
+
+from ranger.core.linemode import (
+    DEFAULT_LINEMODE, DefaultLinemode, TitleLinemode,
+    PermissionsLinemode, FileInfoLinemode, MtimeLinemode, SizeMtimeLinemode,
+)
 from ranger.core.shared import FileManagerAware, SettingsAware
 from ranger.ext.shell_escape import shell_escape
-from ranger.ext.spawn import spawn
+from ranger.ext import spawn
 from ranger.ext.lazy_property import lazy_property
 from ranger.ext.human_readable import human_readable
 
-if hasattr(str, 'maketrans'):
-    maketrans = str.maketrans
-else:
-    from string import maketrans
-_unsafe_chars = '\n' + ''.join(map(chr, range(32))) + ''.join(map(chr, range(128, 256)))
-_safe_string_table = maketrans(_unsafe_chars, '?' * len(_unsafe_chars))
-_extract_number_re = re.compile(r'(\d+|\D)')
-_integers = set("0123456789")
+# Python 2 compatibility
+try:
+    maketrans = str.maketrans  # pylint: disable=invalid-name,no-member
+except AttributeError:
+    from string import maketrans  # pylint: disable=no-name-in-module
+
+
+CONTAINER_EXTENSIONS = ('7z', 'ace', 'ar', 'arc', 'bz', 'bz2', 'cab', 'cpio',
+                        'cpt', 'deb', 'dgc', 'dmg', 'gz', 'iso', 'jar', 'msi', 'pkg', 'rar',
+                        'shar', 'tar', 'tbz', 'tgz', 'xar', 'xpi', 'xz', 'zip')
+DOCUMENT_EXTENSIONS = ('cfg', 'css', 'cvs', 'djvu', 'doc', 'docx', 'gnm',
+                       'gnumeric', 'htm', 'html', 'md', 'odf', 'odg', 'odp', 'ods', 'odt', 'pdf',
+                       'pod', 'ps', 'rtf', 'sxc', 'txt', 'xls', 'xlw', 'xml', 'xslx')
+DOCUMENT_BASENAMES = ('bugs', 'bugs', 'changelog', 'copying', 'credits',
+                      'hacking', 'help', 'install', 'license', 'readme', 'todo')
+
+BAD_INFO = '?'
+
+_UNSAFE_CHARS = '\n' + ''.join(map(chr, range(32))) + ''.join(map(chr, range(128, 256)))
+_SAFE_STRING_TABLE = maketrans(_UNSAFE_CHARS, '?' * len(_UNSAFE_CHARS))
+_EXTRACT_NUMBER_RE = re.compile(r'(\d+|\D)')
 
 
 def safe_path(path):
-    return path.translate(_safe_string_table)
+    return path.translate(_SAFE_STRING_TABLE)
 
 
-class FileSystemObject(FileManagerAware, SettingsAware):
-    (basename,
-    relative_path,
-    relative_path_lower,
-    dirname,
-    extension,
-    infostring,
-    path,
-    permissions,
-    stat) = (None,) * 9
+class FileSystemObject(  # pylint: disable=too-many-instance-attributes,too-many-public-methods
+        FileManagerAware, SettingsAware):
+    basename = None
+    relative_path = None
+    infostring = None
+    path = None
+    permissions = None
+    stat = None
 
-    (content_loaded,
-    force_load,
+    content_loaded = False
+    force_load = False
 
-    is_device,
-    is_directory,
-    is_file,
-    is_fifo,
-    is_link,
-    is_socket,
+    is_device = False
+    is_directory = False
+    is_file = False
+    is_fifo = False
+    is_link = False
+    is_socket = False
 
-    accessible,
-    exists,       # "exists" currently means "link_target_exists"
-    loaded,
-    marked,
-    runnable,
-    stopped,
-    tagged,
+    accessible = False
+    exists = False  # "exists" currently means "link_target_exists"
+    loaded = False
+    marked = False
+    runnable = False
+    stopped = False
+    tagged = False
 
-    audio,
-    container,
-    document,
-    image,
-    media,
-    video) = (False,) * 21
+    audio = False
+    container = False
+    document = False
+    image = False
+    media = False
+    video = False
 
     size = 0
 
+    last_load_time = -1
+
     vcsstatus = None
     vcsremotestatus = None
 
-    _linemode = DEFAULT_LINEMODE
     linemode_dict = dict(
         (linemode.name, linemode()) for linemode in
         [DefaultLinemode, TitleLinemode, PermissionsLinemode, FileInfoLinemode,
@@ -95,34 +103,41 @@ class FileSystemObject(FileManagerAware, SettingsAware):
             self.relative_path = self.basename
         else:
             self.relative_path = relpath(path, basename_is_rel_to)
-        self.relative_path_lower = self.relative_path.lower()
-        self.extension = splitext(self.basename)[1].lstrip(extsep) or None
-        self.dirname = dirname(path)
         self.preload = preload
         self.display_data = {}
 
+    def __repr__(self):
+        return "<{0} {1}>".format(self.__class__.__name__, self.path)
+
+    @lazy_property
+    def extension(self):
         try:
             lastdot = self.basename.rindex('.') + 1
-            self.extension = self.basename[lastdot:].lower()
+            return self.basename[lastdot:].lower()
         except ValueError:
-            self.extension = None
+            return None
 
+    @lazy_property
+    def relative_path_lower(self):
+        return self.relative_path.lower()
+
+    @lazy_property
+    def linemode(self):  # pylint: disable=method-hidden
         # Set the line mode from fm.default_linemodes
         for method, argument, linemode in self.fm.default_linemodes:
             if linemode in self.linemode_dict:
                 if method == "always":
-                    self._linemode = linemode
-                    break
-                if method == "path" and argument.search(path):
-                    self._linemode = linemode
-                    break
+                    return linemode
+                if method == "path" and argument.search(self.path):
+                    return linemode
                 if method == "tag" and self.realpath in self.fm.tags and \
                         self.fm.tags.marker(self.realpath) in argument:
-                    self._linemode = linemode
-                    break
+                    return linemode
+        return DEFAULT_LINEMODE
 
-    def __repr__(self):
-        return "<{0} {1}>".format(self.__class__.__name__, self.path)
+    @lazy_property
+    def dirname(self):
+        return dirname(self.path)
 
     @lazy_property
     def shell_escaped_basename(self):
@@ -131,41 +146,51 @@ class FileSystemObject(FileManagerAware, SettingsAware):
     @lazy_property
     def filetype(self):
         try:
-            return spawn(["file", '-Lb', '--mime-type', self.path])
+            return spawn.check_output(["file", '-Lb', '--mime-type', self.path])
         except OSError:
             return ""
 
     @lazy_property
     def basename_natural(self):
-        return [('0', int(s)) if s in _integers else (s, 0)
-                for s in _extract_number_re.split(self.relative_path)]
+        basename_list = []
+        for string in _EXTRACT_NUMBER_RE.split(self.relative_path):
+            try:
+                basename_list += [('0', int(string))]
+            except ValueError:
+                basename_list += [(string, 0)]
+        return basename_list
 
     @lazy_property
     def basename_natural_lower(self):
-        return [('0', int(s)) if s in _integers else (s, 0)
-                for s in _extract_number_re.split(self.relative_path_lower)]
+        basename_list = []
+        for string in _EXTRACT_NUMBER_RE.split(self.relative_path_lower):
+            try:
+                basename_list += [('0', int(string))]
+            except ValueError:
+                basename_list += [(string, 0)]
+        return basename_list
 
     @lazy_property
     def safe_basename(self):
-        return self.basename.translate(_safe_string_table)
+        return self.basename.translate(_SAFE_STRING_TABLE)
 
     @lazy_property
     def user(self):
         try:
             return getpwuid(self.stat.st_uid)[0]
-        except Exception:
+        except KeyError:
             return str(self.stat.st_uid)
 
     @lazy_property
     def group(self):
         try:
             return getgrgid(self.stat.st_gid)[0]
-        except Exception:
+        except KeyError:
             return str(self.stat.st_gid)
 
     for attr in ('video', 'audio', 'image', 'media', 'document', 'container'):
-        exec("%s = lazy_property("
-            "lambda self: self.set_mimetype() or self.%s)" % (attr, attr))
+        exec(  # pylint: disable=exec-used
+            "%s = lazy_property(lambda self: self.set_mimetype() or self.%s)" % (attr, attr))
 
     def __str__(self):
         """returns a string containing the absolute path"""
@@ -179,33 +204,37 @@ class FileSystemObject(FileManagerAware, SettingsAware):
 
     def set_mimetype(self):
         """assign attributes such as self.video according to the mimetype"""
-        basename = self.basename
+        bname = self.basename
         if self.extension == 'part':
-            basename = basename[0:-5]
-        self._mimetype = self.fm.mimetypes.guess_type(basename, False)[0]
+            bname = bname[0:-5]
+        # pylint: disable=attribute-defined-outside-init
+        self._mimetype = self.fm.mimetypes.guess_type(bname, False)[0]
         if self._mimetype is None:
             self._mimetype = ''
+        # pylint: enable=attribute-defined-outside-init
 
         self.video = self._mimetype.startswith('video')
         self.image = self._mimetype.startswith('image')
         self.audio = self._mimetype.startswith('audio')
         self.media = self.video or self.image or self.audio
         self.document = self._mimetype.startswith('text') \
-                or self.extension in DOCUMENT_EXTENSIONS \
-                or self.basename.lower() in DOCUMENT_BASENAMES
+            or self.extension in DOCUMENT_EXTENSIONS \
+            or self.basename.lower() in DOCUMENT_BASENAMES
         self.container = self.extension in CONTAINER_EXTENSIONS
 
+        # pylint: disable=attribute-defined-outside-init
         keys = ('video', 'audio', 'image', 'media', 'document', 'container')
         self._mimetype_tuple = tuple(key for key in keys if getattr(self, key))
 
         if self._mimetype == '':
             self._mimetype = None
+        # pylint: enable=attribute-defined-outside-init
 
     @property
     def mimetype(self):
         try:
             return self._mimetype
-        except Exception:
+        except AttributeError:
             self.set_mimetype()
             return self._mimetype
 
@@ -213,15 +242,15 @@ class FileSystemObject(FileManagerAware, SettingsAware):
     def mimetype_tuple(self):
         try:
             return self._mimetype_tuple
-        except Exception:
+        except AttributeError:
             self.set_mimetype()
             return self._mimetype_tuple
 
-    def mark(self, boolean):
+    def mark(self, _):
         directory = self.fm.get_directory(self.dirname)
         directory.mark_item(self)
 
-    def _mark(self, boolean):
+    def mark_set(self, boolean):
         """Called by directory.mark_item() and similar functions"""
         self.marked = bool(boolean)
 
@@ -230,26 +259,29 @@ class FileSystemObject(FileManagerAware, SettingsAware):
         if self.is_link:
             try:
                 return realpath(self.path)
-            except Exception:
+            except OSError:
                 return None  # it is impossible to get the link destination
         return self.path
 
-    def load(self):
+    def load(self):  # pylint: disable=too-many-statements
         """Loads information about the directory itself.
 
         reads useful information about the filesystem-object from the
         filesystem and caches it for later use
         """
 
+        self.loaded = True
+        if self.settings.freeze_files:
+            return
+
         self.display_data = {}
         self.fm.update_preview(self.path)
-        self.loaded = True
 
         # Get the stat object, either from preload or from [l]stat
         self.permissions = None
         new_stat = None
         path = self.path
-        is_link = False
+        self.is_link = False
         if self.preload:
             new_stat = self.preload[1]
             self.is_link = new_stat.st_mode & 0o170000 == 0o120000
@@ -264,7 +296,7 @@ class FileSystemObject(FileManagerAware, SettingsAware):
                 if self.is_link:
                     new_stat = stat(path)
                 self.exists = True
-            except Exception:
+            except OSError:
                 self.exists = False
 
         # Set some attributes
@@ -272,16 +304,16 @@ class FileSystemObject(FileManagerAware, SettingsAware):
         self.accessible = True if new_stat else False
         mode = new_stat.st_mode if new_stat else 0
 
-        format = mode & 0o170000
-        if format == 0o020000 or format == 0o060000:  # stat.S_IFCHR/BLK
+        fmt = mode & 0o170000
+        if fmt == 0o020000 or fmt == 0o060000:  # stat.S_IFCHR/BLK
             self.is_device = True
             self.size = 0
             self.infostring = 'dev'
-        elif format == 0o010000:  # stat.S_IFIFO
+        elif fmt == 0o010000:  # stat.S_IFIFO
             self.is_fifo = True
             self.size = 0
             self.infostring = 'fifo'
-        elif format == 0o140000:  # stat.S_IFSOCK
+        elif fmt == 0o140000:  # stat.S_IFSOCK
             self.is_socket = True
             self.size = 0
             self.infostring = 'sock'
@@ -296,6 +328,7 @@ class FileSystemObject(FileManagerAware, SettingsAware):
             self.infostring = '->' + self.infostring
 
         self.stat = new_stat
+        self.last_load_time = time()
 
     def get_permission_string(self):
         if self.permissions is not None:
@@ -327,7 +360,7 @@ class FileSystemObject(FileManagerAware, SettingsAware):
             self.load()
             return True
         try:
-            real_ctime = lstat(self.path).st_ctime
+            real_ctime = stat(self.path).st_ctime
         except OSError:
             real_ctime = None
         if not self.stat or self.stat.st_ctime != real_ctime:
@@ -335,5 +368,5 @@ class FileSystemObject(FileManagerAware, SettingsAware):
             return True
         return False
 
-    def _set_linemode(self, mode):
-        self._linemode = mode
+    def set_linemode(self, mode):
+        self.linemode = mode
